import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class PJMConnectDownload {

    public static void main(String[] args) {
        // PJM Connect credentials
        String username = "jonruffingm2";
        String password = "Opera.123456";

        // Destination folder for downloaded files
        String outputFolder = "C:\\Users\\ruffij\\Desktop\\ConnectIntegration\\ConnectTestPython\\Downloads\\";

        // Optional Search Options
        String rootFolder = ""; // Set to "" to start at topmost level
        boolean recursiveSearch = true; // Set to true to iterate through each folder

        // Connect site URL
        String domain = "https://connectapis.pjm.com/";
        String site = "sites/ftp";

        // Connect library that contains the files
        String libraryName = "File Transfers";

        // Create a QName for the Authentication service
        QName authQName = new QName("http://schemas.microsoft.com/sharepoint/soap/", "Authentication");

        // Create a QName for the Lists service
        QName listsQName = new QName("http://schemas.microsoft.com/sharepoint/soap/", "Lists");

        // Create a QName for the Copy service
        QName copyQName = new QName("http://schemas.microsoft.com/sharepoint/soap/", "Copy");

        try {
            // Create URL for SOAP calls
            URL authUrl = new URL(domain + site + "/_vti_bin/Authentication.asmx?WSDL");
            URL listsUrl = new URL(domain + site + "/_vti_bin/Lists.asmx?WSDL");
            URL copyUrl = new URL(domain + site + "/_vti_bin/Copy.asmx?WSDL");

            // Create service and port for Authentication
            Service authService = Service.create(authUrl, authQName);
            Authentication authentication = authService.getPort(Authentication.class);
            BindingProvider authBindingProvider = (BindingProvider) authentication;
            authBindingProvider.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, username);
            authBindingProvider.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, password);

            // Call the Login method
            authentication.login();

            // Get session cookie for subsequent calls
            String cookie = (String) authBindingProvider.getResponseContext().get("javax.xml.ws.session.maintain");

            // Create service and port for Lists
            Service listsService = Service.create(listsUrl, listsQName);
            Lists lists = listsService.getPort(Lists.class);
            BindingProvider listsBindingProvider = (BindingProvider) lists;
            listsBindingProvider.getRequestContext().put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);
            listsBindingProvider.getRequestContext().put("Cookie", cookie);

            // Build query for GetListItems SOAP call
            String queryOptionsRecursive = "<ViewAttributes Scope=\"RecursiveAll\"/>";
            if (!recursiveSearch) {
                queryOptionsRecursive = "<ViewAttributes Scope=\"All\"/>";
            }
            String queryOptionsFolder = "";
            if (!rootFolder.isEmpty()) {
                queryOptionsFolder = domain + rootFolder;
            }
            String queryOptionsXml = "<QueryOptions><Folder>" + queryOptionsFolder + "</Folder>" + queryOptionsRecursive + "</QueryOptions>";

            // Call the GetListItems method with appropriate parameters
            Element result = lists.getListItems(libraryName, 0, null, queryOptionsXml);

            // Parse the result to retrieve items
            NodeList items = result.getElementsByTagName("z:row");
            for (int i = 0; i < items.getLength(); i++) {
                Element item = (Element) items.item(i);
                String file_Id = item.getAttribute("ows_ID");
                String file_name = item.getAttribute("ows_FileLeafRef").split("#")[1];
                String file_url = domain + item.getAttribute("ows_FileRef").split("#")[1];
                String file_type = item.getAttribute("ows_FSObjType").split("#")[1];

                // Filter files from results. 0 is files and 1 is folders
                if (file_type.equals("0")) {
                    // Retrieve file binary and write file
                    URL fileUrl = new URL(file_url);
                    InputStream inputStream = fileUrl.openStream();
                    Path filePath = Paths.get(outputFolder, file_name);
                    Files.copy(inputStream, filePath);
                    inputStream.close();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}

// Define Authentication interface
interface Authentication {
    void login();
}

// Define Lists interface
interface Lists {
    Element getListItems(String listName, int rowLimit, String query, String queryOptions);
}
